package app.fedilab.fedilabtube.drawer;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.Html;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.method.LinkMovementMethod;
import android.text.style.ForegroundColorSpan;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.PopupMenu;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.content.ContextCompat;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.ViewModelProvider;
import androidx.lifecycle.ViewModelStoreOwner;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import app.fedilab.fedilabtube.PeertubeActivity;
import app.fedilab.fedilabtube.R;
import app.fedilab.fedilabtube.ShowAccountActivity;
import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.RetrofitPeertubeAPI;
import app.fedilab.fedilabtube.client.data.CommentData.Comment;
import app.fedilab.fedilabtube.client.entities.Report;
import app.fedilab.fedilabtube.helper.EmojiHelper;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.viewmodel.PostActionsVM;
import es.dmoral.toasty.Toasty;
import studio.carbonylgroup.textfieldboxes.ExtendedEditText;

import static app.fedilab.fedilabtube.client.RetrofitPeertubeAPI.ActionType.MUTE;
import static app.fedilab.fedilabtube.client.RetrofitPeertubeAPI.ActionType.REPLY;
import static app.fedilab.fedilabtube.helper.Helper.isLoggedIn;


public class CommentListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {


    private final List<Comment> comments;
    private final CommentListAdapter commentListAdapter;
    public AllCommentRemoved allCommentRemoved;
    boolean isVideoOwner;
    private Context context;

    public CommentListAdapter(List<Comment> comments, boolean isVideoOwner) {
        this.comments = comments;
        commentListAdapter = this;
        this.isVideoOwner = isVideoOwner;
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return comments.size();
    }


    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {

        context = parent.getContext();
        LayoutInflater layoutInflater = LayoutInflater.from(this.context);
        return new ViewHolder(layoutInflater.inflate(R.layout.drawer_comment, parent, false));
    }

    @SuppressLint({"SetJavaScriptEnabled", "ClickableViewAccessibility"})
    @Override
    public void onBindViewHolder(@NonNull final RecyclerView.ViewHolder viewHolder, int i) {

        context = viewHolder.itemView.getContext();

        final ViewHolder holder = (ViewHolder) viewHolder;

        final Comment comment = comments.get(i);


        if (comment == null)
            return;
        holder.main_container.setTag(i);

        holder.more_actions.setOnClickListener(view -> {
            PopupMenu popup = new PopupMenu(context, holder.more_actions);
            popup.getMenuInflater()
                    .inflate(R.menu.comment_menu, popup.getMenu());
            if (!Helper.isOwner(context, comment.getAccount())) {
                popup.getMenu().findItem(R.id.action_delete).setVisible(false);
            } else {
                popup.getMenu().findItem(R.id.action_mute).setVisible(false);
                popup.getMenu().findItem(R.id.action_remove_comments).setVisible(false);
                popup.getMenu().findItem(R.id.action_report).setVisible(false);
            }
            if (!isVideoOwner) {
                popup.getMenu().findItem(R.id.action_remove_comments).setVisible(false);
            }
            if (!Helper.isLoggedIn(context)) {
                popup.getMenu().findItem(R.id.action_mute).setVisible(false);
                popup.getMenu().findItem(R.id.action_remove_comments).setVisible(false);
                popup.getMenu().findItem(R.id.action_delete).setVisible(false);
            }
            popup.setOnMenuItemClickListener(item -> {
                int itemId = item.getItemId();
                if (itemId == R.id.action_delete) {
                    AlertDialog.Builder builder = new AlertDialog.Builder(context);
                    builder.setTitle(R.string.delete_comment);
                    builder.setMessage(R.string.delete_comment_confirm);
                    builder.setIcon(android.R.drawable.ic_dialog_alert)
                            .setPositiveButton(R.string.delete, (dialog, which) -> {
                                new Thread(() -> {
                                    new RetrofitPeertubeAPI(context).post(RetrofitPeertubeAPI.ActionType.PEERTUBEDELETECOMMENT, comment.getVideoId(), comment.getId());
                                    Handler mainHandler = new Handler(Looper.getMainLooper());
                                    Runnable myRunnable = () -> {
                                        comments.remove(comment);
                                        notifyDataSetChanged();
                                        if (comments.size() == 0) {
                                            allCommentRemoved.onAllCommentRemoved();
                                        }
                                    };
                                    mainHandler.post(myRunnable);
                                }).start();

                                dialog.dismiss();
                            })
                            .setNegativeButton(R.string.no, (dialog, which) -> dialog.dismiss())
                            .show();
                } else if (itemId == R.id.action_report) {
                    reportComment(comment);
                } else if (itemId == R.id.action_mute) {
                    PostActionsVM viewModel = new ViewModelProvider((ViewModelStoreOwner) context).get(PostActionsVM.class);
                    viewModel.post(MUTE, comment.getAccount().getAcct(), null).observe((LifecycleOwner) context, apiResponse -> manageVIewPostActions(MUTE, 0, apiResponse));
                    comments.remove(comment);
                    notifyDataSetChanged();
                    if (comments.size() == 0) {
                        allCommentRemoved.onAllCommentRemoved();
                    }
                } else if (itemId == R.id.action_remove_comments) {
                    AlertDialog.Builder builder;
                    builder = new AlertDialog.Builder(context);
                    builder.setTitle(R.string.delete_account_comment);
                    builder.setMessage(R.string.delete_account_comment_confirm);
                    builder.setIcon(android.R.drawable.ic_dialog_alert)
                            .setPositiveButton(R.string.delete, (dialog, which) -> {
                                new Thread(() -> {
                                    new RetrofitPeertubeAPI(context).post(RetrofitPeertubeAPI.ActionType.PEERTUBE_DELETE_ALL_COMMENT_FOR_ACCOUNT, comment.getAccount().getAcct(), "my-videos");
                                    Handler mainHandler = new Handler(Looper.getMainLooper());
                                    Runnable myRunnable = () -> {
                                        comments.remove(comment);
                                        notifyDataSetChanged();
                                        if (comments.size() == 0) {
                                            allCommentRemoved.onAllCommentRemoved();
                                        }
                                    };
                                    mainHandler.post(myRunnable);
                                }).start();

                                dialog.dismiss();
                            })
                            .setNegativeButton(R.string.no, (dialog, which) -> dialog.dismiss())
                            .show();
                }
                return true;
            });
            popup.show();
        });
        holder.comment_content.setOnTouchListener((view, motionEvent) -> {
            if (motionEvent.getAction() == MotionEvent.ACTION_UP && !view.hasFocus()) {
                try {
                    view.requestFocus();
                } catch (Exception ignored) {
                }
            }
            return false;
        });

        Spanned commentSpan;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
            commentSpan = Html.fromHtml(EmojiHelper.shortnameToUnicode(comment.getText()), Html.FROM_HTML_MODE_COMPACT);
        else
            commentSpan = Html.fromHtml(EmojiHelper.shortnameToUnicode(comment.getText()));
        holder.comment_content.setText(commentSpan, TextView.BufferType.SPANNABLE);

        holder.comment_content.setMovementMethod(LinkMovementMethod.getInstance());

        holder.comment_account_displayname.setText(comment.getAccount().getDisplayName());

        if (context instanceof PeertubeActivity && !comment.isReply()) {
            holder.main_container.setOnClickListener(v -> ((PeertubeActivity) context).openCommentThread(comment, i));
            holder.comment_content.setOnClickListener(v -> ((PeertubeActivity) context).openCommentThread(comment, i));
        }
        if (comment.getTotalReplies() > 0) {
            holder.number_of_replies.setVisibility(View.VISIBLE);
            holder.number_of_replies.setText(context.getResources().getQuantityString(R.plurals.number_of_replies, comment.getTotalReplies(), comment.getTotalReplies()));
        } else {
            holder.number_of_replies.setVisibility(View.GONE);
        }

        if (comment.getAccount() != null) {
            Spannable wordtoSpan;
            Pattern hashAcct;
            wordtoSpan = new SpannableString("@" + comment.getAccount().getAcct());
            hashAcct = Pattern.compile("(@" + comment.getAccount().getAcct() + ")");
            Matcher matcherAcct = hashAcct.matcher(wordtoSpan);
            while (matcherAcct.find()) {
                int matchStart = matcherAcct.start(1);
                int matchEnd = matcherAcct.end();
                if (wordtoSpan.length() >= matchEnd && matchStart < matchEnd) {
                    wordtoSpan.setSpan(new ForegroundColorSpan(ContextCompat.getColor(context, android.R.color.darker_gray)), matchStart, matchEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                }
            }
            holder.comment_account_username.setText(wordtoSpan);
        }

        holder.comment_date.setText(Helper.dateDiff(context, comment.getCreatedAt()));


        Helper.loadGiF(context, comment.getAccount().getAvatar() != null ? comment.getAccount().getAvatar().getPath() : null, holder.comment_account_profile);

        holder.comment_account_profile.setOnClickListener(v -> {
            Bundle b = new Bundle();
            Intent intent = new Intent(context, ShowAccountActivity.class);
            b.putParcelable("account", comment.getAccount());
            b.putString("accountAcct", comment.getAccount().getAcct());
            intent.putExtras(b);
            context.startActivity(intent);
        });
        if (comment.isReply()) {
            holder.replyButton.setVisibility(View.VISIBLE);
        } else {
            holder.replyButton.setVisibility(View.GONE);
        }
        if (comment.isReply() && comment.isReplyViewOpen()) {
            holder.write_comment_container_reply.setVisibility(View.VISIBLE);
        } else {
            holder.write_comment_container_reply.setVisibility(View.GONE);
        }
        if (holder.add_comment_write_reply.getText() == null || holder.add_comment_write_reply.getText().toString().trim().length() == 0) {
            holder.add_comment_write_reply.setText(String.format("@%s ", comment.getAccount().getAcct()));
            holder.add_comment_write_reply.setSelection(holder.add_comment_write_reply.getText().length());
        }
        holder.replyButton.setOnClickListener(v -> {
            comment.setReplyViewOpen(!comment.isReplyViewOpen());
            notifyItemChanged(i);
        });

        holder.send_reply.setOnClickListener(null);
        holder.send_reply.setOnClickListener(v -> {
            if (isLoggedIn(context)) {
                String commentView = holder.add_comment_write_reply.getText().toString();
                if (commentView.trim().length() > 0) {
                    PostActionsVM viewModelComment = new ViewModelProvider((ViewModelStoreOwner) context).get(PostActionsVM.class);
                    viewModelComment.comment(REPLY, comment.getVideoId(), comment.getId(), commentView).observe((LifecycleOwner) context, apiResponse1 -> manageVIewPostActions(REPLY, (int) holder.main_container.getTag(), apiResponse1));
                    holder.add_comment_write_reply.setText("");
                    comment.setReplyViewOpen(false);
                    notifyItemChanged(i);
                }
            }
        });
    }

    public void manageVIewPostActions(RetrofitPeertubeAPI.ActionType statusAction, int i, APIResponse apiResponse) {

        if (apiResponse.getError() != null) {
            Toasty.error(context, apiResponse.getError().getError(), Toast.LENGTH_LONG).show();
            return;
        }
        if (statusAction == RetrofitPeertubeAPI.ActionType.PEERTUBEDELETECOMMENT) {
            int position = 0;
            for (Comment comment : comments) {
                if (comment.getId().equals(apiResponse.getTargetedId())) {
                    comments.remove(comment);
                    commentListAdapter.notifyItemRemoved(position);
                    break;
                }
                position++;
            }
        } else if (statusAction == REPLY) {
            if (apiResponse.getComments() != null && apiResponse.getComments().size() > 0) {
                comments.add(i + 1, apiResponse.getComments().get(0));
                notifyItemInserted(i + 1);
            }
        } else if (statusAction == RetrofitPeertubeAPI.ActionType.REPORT_COMMENT) {
            Toasty.success(context, context.getString(R.string.successful_report_comment), Toasty.LENGTH_LONG).show();
        } else if (statusAction == MUTE) {
            Toasty.info(context, context.getString(R.string.muted_done), Toast.LENGTH_LONG).show();
        }
    }


    private void reportComment(Comment comment) {
        androidx.appcompat.app.AlertDialog.Builder dialogBuilder = new androidx.appcompat.app.AlertDialog.Builder(context);
        LayoutInflater inflater = ((Activity) context).getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.popup_report, new LinearLayout(context), false);
        dialogBuilder.setView(dialogView);
        EditText report_content = dialogView.findViewById(R.id.report_content);
        dialogBuilder.setNeutralButton(R.string.cancel, (dialog, id) -> dialog.dismiss());
        dialogBuilder.setPositiveButton(R.string.report, (dialog, id) -> {
            if (report_content.getText().toString().trim().length() == 0) {
                Toasty.info(context, context.getString(R.string.report_comment_size), Toasty.LENGTH_LONG).show();
            } else {
                PostActionsVM viewModel = new ViewModelProvider((ViewModelStoreOwner) context).get(PostActionsVM.class);
                Report report = new Report();
                Report.CommentReport commentReport = new Report.CommentReport();
                commentReport.setId(comment.getId());
                report.setComment(commentReport);
                report.setReason(report_content.getText().toString());
                viewModel.report(report).observe((LifecycleOwner) context, apiResponse -> manageVIewPostActions(RetrofitPeertubeAPI.ActionType.REPORT_COMMENT, 0, apiResponse));
                dialog.dismiss();
            }
        });
        androidx.appcompat.app.AlertDialog alertDialog2 = dialogBuilder.create();
        alertDialog2.show();
    }

    public interface AllCommentRemoved {
        void onAllCommentRemoved();
    }

    static class ViewHolder extends RecyclerView.ViewHolder {

        TextView comment_content;
        TextView comment_account_username;
        TextView comment_account_displayname;
        ImageView comment_account_profile, send_reply;
        TextView comment_date, replyButton;
        LinearLayout main_container;
        TextView more_actions, number_of_replies;
        ExtendedEditText add_comment_write_reply;
        ConstraintLayout write_comment_container_reply;


        @SuppressLint("SetJavaScriptEnabled")
        ViewHolder(View itemView) {
            super(itemView);
            comment_content = itemView.findViewById(R.id.comment_content);
            comment_account_username = itemView.findViewById(R.id.comment_account_username);
            comment_account_profile = itemView.findViewById(R.id.comment_account_profile);
            comment_account_displayname = itemView.findViewById(R.id.comment_account_displayname);
            comment_date = itemView.findViewById(R.id.comment_date);
            main_container = itemView.findViewById(R.id.main_container);
            more_actions = itemView.findViewById(R.id.more_actions);
            number_of_replies = itemView.findViewById(R.id.number_of_replies);
            add_comment_write_reply = itemView.findViewById(R.id.add_comment_write_reply);
            send_reply = itemView.findViewById(R.id.send_reply);
            replyButton = itemView.findViewById(R.id.replyButton);
            write_comment_container_reply = itemView.findViewById(R.id.write_comment_container_reply);

        }

        public View getView() {
            return itemView;
        }


    }

}